# Generated by Django 4.2.1 on 2023-07-12 02:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('content_blocks', '0003_alter_contentblocktemplatefield_unique_together_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='contentblocktemplate',
            name='no_cache',
        ),
    ]
