/*CSS for Content Block Template admin change page*/

/* Choices widget */

.choices-widget {
  padding-left: 170px;
}

.choices-widget .inputs {
  clear: left;
}

.choices-widget .input-row {
  padding-bottom: 5px;
}

.choices-widget .input-row input:first-child {
  margin-right: 18px;
}

.choices-widget button {
  outline: none;
  border: none;
  background-color: transparent;
  color: var(--link-fg);
  cursor: pointer;
  font-size: 12px;
  transition:
    color 0.15s,
    background 0.15s;
}
.choices-widget button:hover {
  color: var(--link-hover-color);
}

.choices-widget button.add-choice-row {
  padding-left: 0;
}

/* Smaller help text textarea */
.field-help_text textarea {
  height: 50px;
}

/*Warning when deleting ContentBlockTemplateField*/
.delete-warning {
  display: none;
  float: left;
  font-weight: 400;
  font-size: 0.8125rem;
  color: var(--body-fg);
  padding: 1px 2px 1px 22px;
  margin-right: 4px;
  background: var(--message-warning-bg) url(../../admin/img/icon-alert.svg) 4px 2px no-repeat;
  background-size: 14px auto;
}

input:checked + .delete-warning {
  display: inline;
}
