# Generated by Django 4.2.6 on 2023-10-19 19:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('content_blocks', '0008_auto_20231019_1729'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contentblock',
            name='name',
            field=models.CharField(blank=True, help_text='Used to identify content blocks in the content block editor. Defaults to <content_block_template_name> #<content_block_id>', max_length=320),
        ),
    ]
