# Generated by Django 4.2.6 on 2023-10-19 17:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('content_blocks', '0006_iframefield_contentblockfield_iframe'),
    ]

    operations = [
        migrations.AddField(
            model_name='contentblock',
            name='name',
            field=models.CharField(blank=True, help_text='Used to identify content blocks in the content block editor. Defaults to <content_block_template_name> #<content_block_id>', max_length=64),
        ),
        migrations.AlterField(
            model_name='contentblocktemplatefield',
            name='field_type',
            field=models.CharField(choices=[('TextField', 'Text Field'), ('ContentField', 'Content Field'), ('ImageField', 'Image Field'), ('NestedField', 'Nested Field'), ('CheckboxField', 'Checkbox Field'), ('ChoiceField', 'Choice Field'), ('ModelChoiceField', 'Model Choice Field'), ('FileField', 'File Field'), ('VideoField', 'Video Field'), ('EmbeddedVideoField', 'Embedded Video Field'), ('IframeField', 'Iframe Field')], max_length=32),
        ),
    ]
