<label for="{{ field.auto_id }}" class="{{ field.html_name }} {% if field.is_hidden %}hidden{% endif %} {% if field.field.required %}required{% endif %} fl pos-rel">
  {{ field.label }}:

  {% if field.help_text %}
    <i class="fa-solid fa-light fa-info-circle tooltip help-text pos-abs" title="{{ field.help_text }}"></i>
  {% endif %}
</label>

<div class="field fl clearfix">
  {{ field }}
</div>

{% if field.errors %}
  <div class="field-errors">
    {{ field.errors }}
  </div>
{% endif %}
