<div class="loader pos-abs" id="loader_{{ loader_id }}">
  <div class="bg pos-abs">
    <div class="centab">
      <div class="cencell">
        <div class="cencontent">
          <div class="spinner">
            <i class="fa-solid fa-thin fa-hourglass fa-3x spinner"></i>
          </div>

          <div class="progress">
            <div class="progress-bar"></div>
          </div>
        </div>
      </div>
    </div>
  </div>
</div>
